; Bootsector ok KOS
; version finale ...
; ATTENTION : 
;
;	L'equipe de developpement KOS n'assure aucune garantie quant
;	l'utilisation de ce bootsecteur.
;
; Taille maximum du kernel charge : 512 octets
; Si le kernel devient plus gros (je pense que ce sera vite le cas), mailez-moi
; je vous modifierai ce bootsecteur au plus vite pour qu'il charge l'ensemble
; du kernel...

; Mailez a Thomas PETAZZONI : peta@club-internet.fr

; But global de ce bootsecteur :
;
;		- Initialiser la becane
;		- Charger le kernel
;		- Passer en mode protege
;		- Executer le kernel

; Taille restante :
; Je vous rappelle qu'un bootsecteur ne peut faire qu'au maximum 512 octets
; dont 2 octets obligatoires 0xAA55.
; Sur les 510 octets reellement utilisables, 43,3% sont encore dispo pour du 
; code, soit exactement 221 octets.
;
; Ce qu'il reste a faire :
;
;		- ouvrir la porte A20
;		- procedure de chargement d'un kernel plus gros
;		- si possible un debut d'IDT
;
; Special Thanks to :
;
;	- Emmanuel Marty
;	- Coffeeman
;	- Jerome Petazzoni
;	- Bernard Cassagne


[BITS 16]
[ORG 0x7C00]              ; le bootsecteur est charge a l'adresse
                          ; 0x7C0:0, soit 0x7c00 en lineaire


jmp start                 ; saut vers le debut

bootdrv         db      0 ; on sauvegarde le numero du lecteur sur lequel
                          ; on a boote

; on definit qques messages

texte           db      'Loading KOS', 13, 10 ,0
check           db      'Checking for a 386+ processor...', 0
found386        db      ' Found', 13, 10, 0
need386         db      ' Not Found', 13, 10, 0
diskerror	db	'Disk Error', 13, 10, 0

; Procedure qui affiche a l'ecran la chaine de caracteres pointee par DS:SI

message:
        lodsb           ; charge ds:si dans al et incremente si
        or al, al       ; si al = 0
        jz done         ; alors on arrete car c'est la fin de la chaine
        mov ah, 0eh     ; service 0Eh (affichage d'un caractere)
        mov bx, 0007    ; Parametres : blanc sur fond noir
        int 0x10        ; Appel de l'interruption 10h
        jmp message     ; On repart au dbut ...
done:
        ret             ; si la chaine est finie alors on retourne dans la
                        ; fonction appelante

; Fin de la procedure

no386:
        mov si, need386 ; on fait pointer SI sur le debut de la chaine need386
        call message    ; puis on appelle la fonction message
	jmp reboot

disk_error:			; cette procedure est appelee si il y a une 
	mov si, diskerror	; erreur de disk
	call message        
        
reboot:
        xor ax, ax              ; service 0 
        int 0x16                ; de l'int 16 (attente d'une touche)
        jmp 0xF000:0xFFF0       ; et reboot...
        
start:                  ; c'est ici que tout commence
        
        mov ax, 0       ; on place 0 dans ax
        mov ds, ax      ; et on met donc 0 dans DS (le segment de donnees
                        ; commence a 0x7C00 (voir le ORG) + 0, soit 0x7C00,
			; en lineaire, bien sur...)

        mov [bootdrv], dl       ; on sauvegarde le no du disque sur lequel
                                ; on a boote

; NOTE : On la pile a 0x900:0xf000 ... Ceci est une valeur arbitraire

        
        mov ax, 0x9000          ; on cree une pile..
        mov ss, ax              ; dans le segment 0x900
        mov sp, 0xf000          ; a partir de 0xf000
        

        mov si, texte           ; on affiche 2 messages ... ca fait joli :-)
        call message  
        mov si, check
        call message


; la attention, plus complexe : on teste si le proc est un 386+
; pour cela, on va essayer de modifier les bits 12 ? 14 du registre E-flag
; si la modification reste, alors le proc est un 386+, sinon, c'est =< 286

; Merci a Emmanuel Marty pour la compatibilite avec les 386 "pre-jurassique"

        pushf           ; on sauvegarde le E-Flag
        mov ah, 70h
        push ax
        popf
        pushf
        pop ax
        
        or ah,ah
        je no386        ; si la modif n'est pas valable, alors on saute a
                        ; no386 (voir plus haut)
        popf            ; on les restaure ? la fin ...
        
        mov si, found386 ; si tout est OK, alors on continue en affichant un 
        call message     ; message

; Petite procedure qui charge un petit (512 octets) kernel dans la memoire

readme:                 ; on reset le controleur de disque
        xor ax, ax
        int 0x13
        jc disk_error   ; si il y a une erreur, on affiche un message
			; et on reboote

; on charge donc le secteur 2 (kernel) a l'adresse 0x900:0
; NOTE : Cette adresse est aussi arbitraire

        mov ax, 0x0900
        mov es, ax              ; es = 0x900
        xor bx, bx              ; bx = 0
 
        mov ax, 0x0202          ; params
        mov ch, 0               ; params
        mov cl, 2               ; params
        mov dh, 0               ; params
        mov dl, [bootdrv]       ; no de lecteur
        int 0x13                ; charge le secteur 2
        jc readme               ; si erreur, on reset le controleur
        
; Passage en mode protege

        cli                             ; on disable les interruptions

; Ici on va calculer l'adresse de base de la GDT, qui sera chargee
; dans le GDTRegister (ou GDTR). Cette adresse doti cependant etre lineaire
; donc on multiplie par 16 le segment, et on ajoute l'offset...
; Easy, non ?

        xor eax, eax                    ; EAX = 0;
        mov ax, cs                      ; AX = CS;
        shl eax, 4                      ; AX = CS * 16 (on calcule l'adresse
                                        ; lineaire de CS)
        add [gdt_register+2], eax       ; on update le gdt_register
        lgdt [gdt_register]             ; on charge la GDT

        mov eax, cr0
        or al, 1
        mov cr0, eax                    ; et pouf, nous voila en modep

        mov ax,10h			; selecteur de donnees
        mov ds,ax			; on initialise tous les registres de
        mov es,ax			; segments pour qu'il pointe sur des
        mov fs,ax			; descripteurs de segment de la GDT
        mov gs,ax
        mov ss,ax
        mov esp,9f000h			; Et la pile en 0x9F000

        jmp dword 8h:9000h              ; on saute sur le kernel

gdt:                                    ; c'est la GDT, elle definit les 
                                        ; segments
        dw 0,0,0,0                      ; null descriptor (obligatoire)

cs_seg:
        dw 0x07FF                       ; segment de code de 8 Mo
        dw 0x0000
        db 0x00
        db 0x9A
        db 0xC0
        db 0x00

ds_seg:
        dw 0xFFFF                       ; segment de donnes de 4 Go
        dw 0x0000
        db 0x00
        db 0x92
        db 0xCF
        db 0x00

gdt_register:
        dw 0x7ff                        ; 48 bits, charges dans le regsitre
        dw gdt, 0                       ; GDTR, par l'instruction LGDT

; Petites instructions pour que le bootsecteur fasse 512 octets dont les deux
; derniers sont 0xAA55

; On balance des NOP (opcode 0x90 = 144) jusqu'a 510 inclus

times 510-($-$$) db 144

; A 511 et 512, on fout 0xAA et 0x55

dw 0xAA55
